/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
/* *
 *
 * Appel dans un programme C des routines spls,predict,calcul_vip de pls4snp
 *
 * auteur : olivier.filangi at rennes.inra.fr
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
using namespace std;

/* include pls4snp */
#include "pls4snp.hpp"

void read_Z(string file, int n,int p, double *Z) {
	string name;
	string snp;
	string line;
	ifstream zfile;
	zfile.open(file.c_str());
	int c=0;
	while ( ! zfile.eof() )
	{
		getline (zfile,line);
		if ( line == "" ) continue;
		istringstream stream1(line);
		stream1 >> name;
		stream1 >> snp;
		for (int i=0;i<p;i++) {
			Z[c+i*n] = snp[i] - '0';
		}
		c++;
	}

	zfile.close();
}

void read_Y(string file, double *Y) {
	string name;
	string snp;
	string line;
	ifstream yfile;
	yfile.open(file.c_str());
	int c=0;

	getline (yfile,line);/* header to remove */

	while ( ! yfile.eof() )
	{
		getline (yfile,line);
		if ( line == "" ) continue;
		istringstream stream1(line);
		stream1 >> name;
		stream1 >> Y[c];
		c++;
	}

	yfile.close();
}


int main( int argc, const char* argv[] )
{
	cout << "Program C++ / Appel à libpls4snp" << endl ;

	int ncomp = 10;       // number of latent variables to build
	int p=9990;           // number of marker
	int n_ap = 1500;      // number of reference population
	int n_val=500;        // number of validation population
	int n_pred=1000;      // number of animal to predict
	int maxiter=20;       // max iteration for convergence
	double tol = 0.00001; // criteria convergence
	int keepx = -1;       // if spls, set the number of snp to keep

	double *Z_ap = new double [n_ap*p];
	double *Z_val = new double [n_val*p];
	double *Y_ap = new double [n_ap];
	double *Y_val = new double [n_val];

	read_Z("../typage_ap",n_ap,p,Z_ap);
	read_Z("../typage_val",n_val,p,Z_val);
	read_Y("../pheno_ap.txt",Y_ap);
	read_Y("../pheno_val.txt",Y_val);

	/* init for latent variates and buf struct for pls */
	double * t = new double[n_ap*ncomp];
	double * u = new double[n_ap*ncomp];
	double * a = new double[p*ncomp];
	double * c = new double[p*ncomp];
	double * b = new double[ncomp];
	double * meanZ = new double[p];
	double * sigmZ = new double[p];

	double meanY,sigmaY;
	int edc_pres=1; // false
	double * EDC = NULL ;

	/* Lib PLS4SNP : build Latent Variables with PLS */

	spls_(&n_ap,&p,Z_ap,Y_ap,&edc_pres,EDC,&ncomp,&maxiter,&tol,&keepx,
			&meanY,&sigmaY,meanZ,sigmZ,t,u,a,b,c);

	double * yhat = new double[n_val*ncomp];
	double * tpred = new double[n_val*ncomp];
	double * bhat = new double[p*ncomp];

	/* Lib PLS4SNP : Predict Y for successive combination of latent variables */
	predict_(&n_ap,&p,&ncomp,a,c,b,&meanY,&sigmaY,meanZ,sigmZ,&n_val,Z_val,&ncomp,yhat,tpred,bhat);

	/** Correlation between YVAL and each YPRED **/
	double cormax = 0;
	int hmax = 0;

	/* mean YVAL */
	double meanYVAL = 0;
	for (int i=0;i<n_val;i++)
		meanYVAL += Y_val[i];
	meanYVAL /= n_val;

	/* Temporary variable for correlation computation */
	double * aa = new double [n_val];
	for (int i=0;i<n_val;i++)
		aa[i] = Y_val[i] - meanYVAL ;

	double * bb = new double [n_val];
	for (int h=0;h<ncomp;h++) {
		/* mean YPRED */
		double meanYPRED = 0;
		for (int i=0;i<n_val;i++)
			meanYPRED += yhat[i+h*n_val]; /* CARREFULLY : Fortran access - Column Major Order */
		meanYPRED /= n_val;

		for (int i=0;i<n_val;i++)
			bb[i] = yhat[i+h*n_val] - meanYPRED ;

		double saa=0,sab=0,sbb=0;
		for (int i=0;i<n_val;i++) {
			saa += aa[i]*aa[i];
			sab += aa[i]*bb[i];
			sbb += bb[i]*bb[i];
		}

		double cor = sab / sqrt(saa*sbb);
		if (cor > cormax) {
			cormax = cor;
			hmax = h;
		}
		cout << "H=" << (h+1) << " correlation="<< cor << endl;
	}

	cout << "Hmax="<< (hmax+1) << endl ;
	delete[]aa;
	delete[]bb;

	int hmaxborn=hmax+1;
	double * VIP = new double [hmaxborn*p];

	/* Lib PLS4SNP : Information about SNP effect : VIP computation */
	calcul_vip_(&ncomp,&hmaxborn,&n_ap,&p,Y_ap,t,a,VIP);

	cout << " == Print VIP in file : vip.txt ==" << endl ;
	ofstream vipfile;
	vipfile.open("vip.txt");
	vipfile.precision(5);

	for (int i=0;i<p;i++) {
		vipfile << VIP[i+hmax*p]<< endl ;
	}

	vipfile.close();
	delete[] VIP;
	delete[] yhat;
	delete[] tpred;
	delete[] bhat;

	/* Predict Y with hmax */

	double *Z_pred = new double [n_pred*p];
	double *Y_pred = new double [n_pred];

	read_Z("../typage_pred",n_pred,p,Z_pred);
	read_Y("../tbv.txt",Y_pred);

	yhat = new double[n_pred*hmaxborn];
	tpred = new double[n_pred*hmaxborn];
	bhat = new double[p*hmaxborn];

	/* Lib PLS4SNP : Predict Y for successive combination of latent variables */
	predict_(&n_ap,&p,&ncomp,a,c,b,&meanY,&sigmaY,meanZ,sigmZ,&n_pred,Z_pred,&hmaxborn,yhat,tpred,bhat);

	/* Solution in the matrix yhat at hmax column */

	/* Correlation with TBV */
	/* mean YVAL */
	meanYVAL = 0;
	for (int i=0;i<n_pred;i++)
		meanYVAL += Y_pred[i];
	meanYVAL /= n_pred;

	/* Temporary variable for correlation computation */
	aa = new double [n_pred];
	bb = new double [n_pred];
	for (int i=0;i<n_pred;i++)
		aa[i] = Y_pred[i] - meanYVAL ;
	/* mean YPRED */
	double meanYPRED = 0;
	for (int i=0;i<n_pred;i++)
		meanYPRED += yhat[i+hmax*n_pred]; /* CARREFULLY : Fortran access - Column Major Order */
	meanYPRED /= n_pred;
	for (int i=0;i<n_pred;i++)
		bb[i] = yhat[i+hmax*n_pred] - meanYPRED ;
	double saa=0,sab=0,sbb=0;
	for (int i=0;i<n_pred;i++) {
		saa += aa[i]*aa[i];
		sab += aa[i]*bb[i];
		sbb += bb[i]*bb[i];
	}

	double cor = sab / sqrt(saa*sbb);

	cout << "Correlation between true breading value and prediction:"<< cor << endl ;

	delete[] yhat;
	delete[] tpred;
	delete[] bhat;

	delete[] meanZ;
	delete[] sigmZ;

	delete[] t;
	delete[] u;
	delete[] a;
	delete[] c;
	delete[] b;

	delete[] Z_ap;
	delete[] Z_val;
	delete[] Y_ap;
	delete[] Y_val;
	delete[] Z_pred;
	delete[] Y_pred;
}
