!/*
! *   This file is part of pls4snp.
! *
! *   pls4snp is free software; you can redistribute it and/or modify
! *   it under the terms of the GNU General Public License as published by
! *   the Free Software Foundation; either version 2 of the License, or
! *   (at your option) any later version.
! *
! *   pls4snp is distributed in the hope that it will be useful,
! *   but WITHOUT ANY WARRANTY; without even the implied warranty of
! *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! *   GNU General Public License for more details.
! *
! *   You should have received a copy of the GNU General Public License
! *   along with pls4snp; if not, write to the Free Software
! *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
! *
! *   Copyright 2013 Olivier Filangi
! *
! */
!
!
! Exemple d'appel des subroutine spls, predict et calcul_vip à partir d'un programme fortran
!
! olivier.filangi at rennes.inra.fr
!
! revision 20 Mars 2013
!
program pls_prog
    implicit none

    interface
            subroutine spls(n, p,Z,Y,edc_present,EDC,ncomp,maxiter,tol,keepX,meanY,sigmY,meanZ,sigmZ,t,u,a,b,c)
               integer ,intent(in)                        :: n
               integer ,intent(in)                        :: p
               double precision,dimension(n,p),intent(in) :: Z
               double precision,dimension(n),intent(in)   :: Y
               integer ,intent(in)                        :: edc_present
               double precision,dimension(n),intent(in)   :: EDC
               integer ,intent(in)                        :: ncomp
               integer ,intent(in)                        :: maxiter
               double precision ,intent(in)               :: tol
               integer ,intent(in)                        :: keepX
               double precision ,intent(out)              :: meanY
               double precision ,intent(out)              :: sigmY
               double precision,dimension(p),intent(out)  :: meanZ
               double precision,dimension(p),intent(out)  :: sigmZ
               double precision,dimension(n,ncomp),intent(out)  :: t
               double precision,dimension(n,ncomp),intent(out)  :: u
               double precision,dimension(p,ncomp),intent(out)  :: a
               double precision,dimension(ncomp),intent(out)    :: b
               double precision,dimension(p,ncomp),intent(out)  :: c
            end subroutine spls

           subroutine calcul_vip(ncomp,hcomp,n, p,Y,t,a,VIP)
              integer ,intent(in)                              :: ncomp
              integer ,intent(in)                              :: hcomp
              integer ,intent(in)                              :: n
              integer ,intent(in)                              :: p
              double precision,dimension(n),intent(in)         :: Y
              double precision,dimension(n,ncomp),intent(in)   :: t
              double precision,dimension(p,ncomp),intent(in)   :: a
              double precision,dimension(p,hcomp),intent(out)  :: VIP
           end subroutine calcul_vip

           subroutine predict(n,p,ncomp,a,c,b,meanY,sigmaY,meanZ,sigmaZ,n_pred,Z_pred,hcomp,yhat,tpred,bhat)
              integer ,intent(in)                              :: n
              integer ,intent(in)                              :: p
              integer ,intent(in)                              :: ncomp
              double precision,dimension(p,ncomp),intent(in)   :: a
              double precision,dimension(p,ncomp),intent(in)   :: c
              double precision,dimension(ncomp),intent(in)     :: b
              double precision             ,intent(in)         :: meanY
              double precision             ,intent(in)         :: sigmaY
              double precision,dimension(p)      ,intent(in)   :: meanZ
              double precision,dimension(p)      ,intent(in)   :: sigmaZ
              integer ,intent(in)                              :: n_pred
              double precision,dimension(n_pred,p) ,intent(in) :: Z_pred
              integer ,intent(in)                              :: hcomp
              double precision,dimension(n_pred,hcomp),intent(out)  :: yhat
              double precision,dimension(n_pred,hcomp),intent(out)  :: tpred
              double precision,dimension(p,hcomp),intent(out)       :: bhat
           end subroutine predict

  end interface

  integer             :: hmax
  character(len=9990) :: typeid
  character(len=30)   :: file
  integer             :: i,j,h,ios,n_ap,n_val,n_pred,p,keepx,maxiter,ncomp,edc_present
  ! DATA
  double precision  ,dimension(:,:), allocatable  :: Z_ap,Z_val,Z_pred
  double precision  ,dimension(:)  , allocatable  :: Y_ap,Y_val,Y_tbv
  double precision                                :: val,tol,meanY,sigmY
  ! Results
  double precision  ,dimension(:,:), allocatable  :: t,u,a,c
  double precision  ,dimension(:)  , allocatable  :: b,meanZ,sigmZ,EDC
  double precision  ,dimension(:,:), allocatable  :: yhat,tpred,bhat,VIP
  double precision :: meanYVal,meanYpred,cor,cormax
  double precision  ,dimension(:)  , allocatable  :: AA,BB

  print *,"Program Fortran / Appel à libpls4snp"

  ncomp = 10      ! number of latent variate to build
  maxiter = 20    ! maxiter criteria
  tol = 1.d-5     ! tolerance for pls
  keepx = -1      ! PLS so....
  edc_present = 1 ! no edc
  n_ap = 1500     ! size reference population
  n_val = 500     ! size validation population
  n_pred = 1000   ! size predict population
  p = 9990        ! number of SNP

  allocate (Z_ap(n_ap,p))
  allocate (Y_ap(n_ap))
  allocate (Z_val(n_val,p))
  allocate (Y_val(n_val))

  file='../typage_ap';call read_Z(file,n_ap,p,Z_ap)
  file="../pheno_ap.txt";call read_Y(file,n_ap,Y_ap)
  file='../typage_val';call read_Z(file,n_val,p,Z_val)
  file="../pheno_val.txt";call read_Y(file,n_val,Y_val)

  ! Allocate results structures
  allocate(t(n_ap,ncomp),u(n_ap,ncomp),a(p,ncomp),c(p,ncomp),b(ncomp))
  allocate (meanZ(p),sigmZ(p))

  !Call lib PLS4SNP => spls
  call spls(n_ap,p,Z_ap,Y_ap,edc_present,EDC,ncomp,maxiter,tol,keepX,meanY,sigmY,meanZ,sigmZ,t,u,a,b,c)

  !Allocate prediction results
  allocate(yhat(n_val,ncomp),tpred(n_val,ncomp),bhat(p,ncomp))
  !Call lib PLS4SNP => predict
  call predict(n_ap,p,ncomp,a,c,b,meanY,sigmY,meanZ,sigmZ,n_val,Z_val,ncomp,yhat,tpred,bhat)

  allocate (AA(n_val),BB(n_val))
  meanYVal = sum(Y_val) / n_val
  AA = Y_val - meanYVal
  cormax=0;hmax=0;
  do h=1,ncomp
    meanYpred = sum(yhat(:,h)) / n_val
    print *,meanYpred
    BB = yhat(:,h) - meanYpred
    cor = sum(AA*BB) / sqrt(sum(AA*AA)*sum(BB*BB))
    print *,"H=",h," correlation=",cor
    if (cormax<abs(cor)) then
     cormax = cor
     hmax = h
    end if
  end do

  deallocate (AA,BB)

  print *,"hmax=",hmax
  allocate (VIP(p,hmax))
  !Call lib PLS4SNP => calcul_vip
  call calcul_vip(ncomp,hmax,n_ap,p,Y_ap,t,a,VIP)

  print *," == Print VIP in file : vip.txt =="
  open(unit=20,file="vip.txt")
  do i=1,p
    write(20,fmt="(f7.5)") VIP(i,hmax)
  end do
  close(20)


  deallocate(VIP)
  deallocate(yhat,tpred,bhat)

  allocate (Z_pred(n_pred,p))
  allocate (Y_tbv(n_pred))

  file='../typage_pred';call read_Z(file,n_pred,p,Z_pred)
  file="../tbv.txt";call read_Y(file,n_pred,Y_tbv)

  !Allocate prediction results
  allocate(yhat(n_pred,hmax),tpred(n_pred,hmax),bhat(p,hmax))
  !Call lib PLS4SNP => predict
  call predict(n_ap,p,ncomp,a,c,b,meanY,sigmY,meanZ,sigmZ,n_pred,Z_pred,hmax,yhat,tpred,bhat)
  !read true breading value and compute correlation
  allocate (AA(n_pred),BB(n_pred))
  meanYVal = sum(Y_tbv) / n_pred
  AA = Y_tbv - meanYVal
  meanYpred = sum(yhat(:,hmax)) / n_pred
  BB = yhat(:,hmax) - meanYpred
  cor = sum(AA*BB) / sqrt(sum(AA*AA)*sum(BB*BB))
  print *,"Correlation between true breading value and prediction:",cor

  deallocate (AA,BB)
  deallocate (Z_ap,Y_ap,Z_val,Y_val,Z_pred)
  deallocate(yhat,tpred,bhat)
  deallocate (meanZ,sigmZ)
  deallocate(t,u,a,c,b)

  contains

  subroutine read_Z(file,n,p,Z)
     character(len=30) ,intent(in)   :: file
     integer           ,intent(in)   :: n
     integer           ,intent(in)   :: p
     double precision,dimension(n,p),intent(inout) :: Z
     integer :: ios,j,i,id

     open(unit=19,file=file)
     ios=0
     j=0
     do while( ios == 0 )
       read (19,*,iostat=ios) id,typeid
       if ( ios == 0 ) then
       j=j+1
       do i=1,len(typeid)
        Z(j,i) = ichar(typeid(i:i)) - ichar('0')
       end do
       end if
     end do
     print *,"read ",j," genotypes"
     close(19)

  end subroutine read_Z

  subroutine read_Y(file,n,Y)
     character(len=30) ,intent(in)   :: file
     integer           ,intent(in)   :: n
     double precision,dimension(n),intent(inout) :: Y
     integer :: ios,i,id

     open(unit=19,file=file)
     ios=0
     i=0
     read (19,*,iostat=ios) ! header....
     do while( ios == 0 )
      read (19,*,iostat=ios) id,val
      if ( ios == 0 ) then
        i=i+1
        Y(i) = val
      end if
     end do

     print *,"read ",i," phenotypes - validation "
  close(19)
  end subroutine read_Y

end program pls_prog
