/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
/*
 * CalculDataGen.cpp
 *
 *  Created on: 4 mars 2013
 *      Author: ofilangi
 */

#include "calculDataGen.hpp"

#include <iostream>
#include <string>
#include <map>
using namespace std;

#include <math.h>


CalculDataGen::CalculDataGen() {
	// TODO Auto-generated constructor stub

}

CalculDataGen::~CalculDataGen() {
	// TODO Auto-generated destructor stub
}



/*
 */

void CalculDataGen::scaleWithEDC(dataGen & data){
	cout << "scale Y and X with sqrt(EDC) " << endl ;
	DT * sqrtWT = new DT[data.n];

	for (map<string,DT *>::const_iterator it=data.mapY.begin();it != data.mapY.end();it++) {
		DT * Y = (*it).second;
		map<string,DT *>::const_iterator itEDC = data.mapEDC.find((*it).first);
		if ( itEDC == data.mapEDC.end() ) {
			cout << "pas d edc pour la perf: ["<< (*it).first <<"]" << endl ;
			continue;
		}
		DT *EDC = (*itEDC).second;

		// calcul sum
		DT sumEDC=0;

		for (int i=0;i<data.n;i++) {
			sumEDC += EDC[i];
		}
		//		cout << "sum="<< sumEDC << endl ;
		DT invsum=1/sumEDC;

		for (int i=0;i<data.n;i++) {
			sqrtWT[i] = sqrt(EDC[i] * invsum);
		}
		//		cout << "WT[0]="<< sqrtWT[0]*sqrtWT[0] << endl ;

		//scale Y and Z
		for (int i=0;i<data.n;i++) {
			Y[i] *=  sqrtWT[i] ;
			for (int j=0;j<data.p;j++) {
				data.Z[i][j] *= sqrtWT[i] ;
			}
		}
	}

	delete[] sqrtWT;
}

DT CalculDataGen::correlation_edc(DT * Ypredict,const dataGen & data_ap,string name) {
	DT p=0.0;

	map<string,DT *>::const_iterator itY = data_ap.mapY.find(name) ;
	if ( itY == data_ap.mapY.end() ) {
		cerr << " correlation_edc can not find trait "<< name << endl;
		return p;
	}
	map<string,DT *>::const_iterator itEDC = data_ap.mapEDC.find(name) ;
	if ( itEDC == data_ap.mapEDC.end() ) {
		cerr << " correlation_edc can not find edc for trait "<< name << endl;
		return p;
	}

	DT * Y = (*itY).second ;
	DT * EDC = (*itEDC).second ;

	DT DyDEDCBarre=0.0;
	DT sumEDC = 0.0 ;
	DT DyDHatEDCBarre=0.0;

	for (int i=0;i<data_ap.n;i++) {
		DyDEDCBarre += Y[i]*EDC[i];
		sumEDC += EDC[i];
		DyDHatEDCBarre += Ypredict[i]*EDC[i];
	}
	DyDEDCBarre /= sumEDC;
	DyDHatEDCBarre /= sumEDC;

	p = 0.0;
	DT sumNum=0.0;
	DT sumDenom1=0.0;
	DT sumDenom2=0.0;
	DT a,b;

	for (int i=0;i<data_ap.n;i++) {
		a = Y[i] - DyDEDCBarre;
		b = Ypredict[i] - DyDHatEDCBarre ;
		sumNum += a*b*EDC[i];
		sumDenom1 += a*a*EDC[i];
		sumDenom2 += b*b*EDC[i];
	}
	p = (sumNum / sqrt(sumDenom1*sumDenom2) );

	return p;
}

DT CalculDataGen::correlation(DT * Ypredict,DT * Y,int n) {

	if ( n<= 0 ) return 0;

	DT p=0.0;

	DT DyDEDCBarre=0.0;
	DT DyDHatEDCBarre=0.0;

	for (int i=0;i<n;i++) {
		DyDEDCBarre += Y[i];
		DyDHatEDCBarre += Ypredict[i];
	}
	DyDEDCBarre /= n;
	DyDHatEDCBarre /= n;

	p = 0.0;
	DT sumNum=0.0;
	DT sumDenom1=0.0;
	DT sumDenom2=0.0;
	DT a,b;

	for (int i=0;i<n;i++) {
		a = Y[i] - DyDEDCBarre;
		b = Ypredict[i] - DyDHatEDCBarre ;
		sumNum += a*b;
		sumDenom1 += a*a;
		sumDenom2 += b*b;
	}
	p = (sumNum / sqrt(sumDenom1*sumDenom2) );

	return p;
}
