/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */

/*
 * cli.cpp
 *
 *  Created on: 5 mars 2013
 *      Author: ofilangi
 */
#include <algorithm>
#include <iostream>
#include <time.h>
#include "cli.hpp"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

void cli::print_help(char* progname) {
	string prog = progname;

	unsigned found = prog.rfind("/");
	if (found!=std::string::npos)
		prog = prog.substr(found+1,prog.size());

	cout << "Usage: "<< prog << " [option] "<< endl ;
	cout << "Régression PLS et sparse PLS pour l'analyse des données génomiques."<< endl ;
	cout << endl ;
	cout << "                                    ***                                                   " << endl ;
	cout << endl;
	cout << "  --pls,--spls <int> :"<< "Exécute une régression pls ou sparse pls " << endl ;
	cout << "                       "<< "La spls impose une selection des variables (variables explicatives les plus pertinentes à intégrer dans chaque variable latente)" << endl ;
	cout << endl;
	cout << "          --ncomp <int>         :"<< "nombre de variable latente" << endl ;
	cout << "          --edc                 :"<< "flag si l'EDC est definit dans le fichier de performance" << endl ;
	cout << "              --not-use-edc     :"<< "flag indiquant qu'on utilise pas l'EDC" << endl ;
	cout << "          --maxiter <int>       :"<< "le nombre d'iteration maximum pour la convergence" << endl ;
	cout << "          --tol <float>         :"<< "tolérance pour le critère d'arrêt" << endl ;
	cout << "          --geno <file>         :"<< "fichier de genotype" << endl ;
	cout << "              --header-geno     :"<< "flag indiquant si le fichier de génotype contient une entête" << endl ;
	cout << "              --format-geno  <aipl|uga|mpe|aipl_space|plink> "<< endl ;
	cout << "                                : format du fichier de genotype" << endl ;
	cout << "          --pheno <file>        :"<< "fichier de phenotype" << endl ;
	cout << "          --bin-prefix <string> :"<< "prefix du fichier binaire généré contenant les résultats" << endl ;
	cout << endl;

	cout << "                                    ***                                                   " << endl ;
	cout << endl;

	cout << "  --valid   :"<< "Etape de validation. Calcul la correlation des phenotypes prédits et des phénotypes donnés en argument pour chaque variable latente." << endl ;
	cout << "          --binary <file-bin>   :"<< "fichier binaire à utiliser" << endl ;
	cout << "          --geno <file>         :"<< "fichier de genotype" << endl ;
	cout << "              --header-geno     :"<< "flag indiquant si le fichier de génotype contient une entête" << endl ;
	cout << "              --format-geno  <aipl|uga|mpe|aipl_space|plink> :"<< "format du fichier de genotype" << endl ;
	cout << "          --pheno <file>        :"<< "fichier de phenotype" << endl ;
	cout << endl;

	cout << "                                    ***                                                   " << endl ;
	cout << endl;

	cout << "  --predict :"<< "Exécute la prédiction des phénotypes pour une poulation génotypée." << endl ;
	cout << "          --binary <file-bin>   :"<< "fichier binaire à utiliser" << endl ;
	cout << "          --geno <file>         :"<< "fichier de genotype" << endl ;
	cout << "              --header-geno     :"<< "flag indiquant si le fichier de génotype contient une entête" << endl ;
	cout << "              --format-geno  <aipl|uga|mpe|aipl_space|plink> :"<< "format du fichier de genotype" << endl ;
	cout << "          --out <file>              :"<< "fichier résultat" << endl ;
	cout << "          --H <int>                 :"<< "nombre de variables latentes explicatives" << endl ;
	cout << endl;

	cout << "                                    ***                                                   " << endl ;
	cout << endl;

	cout << "  --vip     :"<< "Calcul des coéfficients VIP permettants de juger l'importance des effets SNP." << endl ;
	cout << "          --bin-prefix <file-bin>   :"<< "fichier binaire à utiliser" << endl ;
	cout << "          --out <file>              :"<< "fichier résultat" << endl ;
	cout << "          --H <int>                 :"<< "nombre de variables latentes explicatives" << endl ;

	cout << endl ;

	cout << endl;
	cout << endl;

	//cout << "        =  "<< " ... " << endl ;
}


cli::cli(int argc, char* argv[]) {
#ifdef HAVE_CONFIG_H
	time_t rawtime;
	struct tm * timeinfo;
	time ( &rawtime );
	timeinfo = localtime ( &rawtime );
	char s[100];
	strftime(s,30,"%b.%d.%Y - %H.%M",timeinfo);
	cout << "Version "<< VERSION << "  Date "<< s << endl ;
#endif
	options.clear();
	nameProg = argv[0];
	bool skeepnext = false;
	set_options_need_values();

	for (int i=1;i<argc;i++) {
		if (skeepnext) {
			skeepnext = false;
			continue;
		}
		string key = argv[i];
		std::vector<string>::const_iterator findIter = std::find(keysValue.begin(), keysValue.end(),key);
		if ( findIter != keysValue.end() ) {
			skeepnext = true;
			if ( i+1 >= argc ) {
				cerr << key << " option need a value!" << endl ;
				exit(1);
			}

			options[key]=argv[i+1];

			if (options[key].find("--",0,2) != std::string::npos ) {
				cerr << key << " option need a value!" << endl ;
				exit(1);
			}
		} else {
			options[key]="ok";
		}
	}
}

cli::~cli() {
	options.clear();
}

void cli::set_options_need_values() {
	keysValue.push_back("--ncomp");
	keysValue.push_back("--bin-prefix");
	keysValue.push_back("--pheno");
	keysValue.push_back("--geno");
	keysValue.push_back("--binary");
	keysValue.push_back("--maxiter");
	keysValue.push_back("--tol");
	keysValue.push_back("--format-geno");
	keysValue.push_back("--H");
	keysValue.push_back("--dev-convgeno");
	keysValue.push_back("--dev-removemaf");
	keysValue.push_back("--spls");
	keysValue.push_back("--out");
	keysValue.push_back("--remove-minmaf");
}

bool cli::keyExist (const string & key) const {
	map<string,string>::const_iterator it = options.find(key);
	return (it != options.end());
}

void cli::check_with_error(const string option,const string & mesg) const {
	if ( ! keyExist(option) ) {
		cerr << "Option : "<< option << endl << mesg << endl ;
		exit(1);
	}
}
