/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
/*
 * command.hpp
 *
 *  Created on: 14 mars 2013
 *      Author: ofilangi
 */

#ifndef COMMAND_HPP_
#define COMMAND_HPP_

#include <string>
using namespace std;

#include "common.hpp"


class Command {

public:

	/* Sparse PLS methods */
	static void command_spls(
			const int nkeep,
			int ncomp,
			int maxiter,
			DT tol,
			const string & phenotypeFile_ap,
			const string & genotypeFile_ap,
			const string & binprefix,
			bool edc_present,
			bool edc_calcul,
			int codeFormatGeno,
			bool headerGeno,
			DT mafmin,
			bool removeMinMaf) ;

	/* PLS methods */
	static void command_pls(
				int ncomp,
				int maxiter,
				DT tol,
				const string & phenotypeFile_ap,
				const string & genotypeFile_ap,
				const string & binprefix,
				bool edc_present,
				bool edc_calcul,
				int codeFormatGeno,
				bool headerGeno,
				DT mafmin,
				bool removeMinMaf) ;

	/* Predict Y (GEBV) */
	static void command_predict(
			const string & genotypeFile_val,
			const string & binary,
			int H,
			int codeFormatGeno,
			bool headerGeno,
			const string & nameOutFile);

	/* Etape de validation
	 * avec une population on calcul les Y prédit et on calcul la correlation avec avec les Y réelles
	 * Une correlation est donnée pour chaque ajout de variable latente
	 */
	static void command_valid(const string & phenotypeFile_val,
			const string & genotypeFile_val,
			const string & binary,
			bool edc_present,
			bool edc_calcul,
			int codeFormatGeno,
			bool headerGeno);

	/* Generation des VIP (effet SNP)*/
	static void command_vip(const string & binary,int H,const string & out) ;

};


#endif /* COMMAND_HPP_ */
