/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
#include <string>
#include <iostream>
using namespace std;

#include "common.hpp"
#include <math.h>


DT calmean(DT * v,int n) {
	double mean=0;
	for (int i=0; i<n; i++)
	{
		mean = mean + v[i];
	}

	mean = mean/(n);
	return mean;
}

DT calsig(DT * v,int n,const DT mean) {
	double sig=0,vv=0,sv=0;
	for (int i=0; i<n; i++)
	{
		vv = (v[i] - mean);
		sv = sv + vv*vv;
	}
	sig= sqrt((sv/(n-1))) ;
	return sig;
}

void standart1(DT * v,int n,DT &mean,DT &sig)
{
	double sv=0;
	//mean
	mean=calmean(v,n);
	sig=0;

	for (int i=0; i<n; i++)
	{
		v[i] = (v[i] - mean);
		sv = sv + v[i]*v[i];
	}

	//sig= sqrt((sv/(n)) - mean*mean) ;
	sig= sqrt((sv/(n-1))) ;
	//sig=1;
	//standart core
	for (int i=0; i<n; i++)
	{
		// v[i] = (v[i] - mean)/sig ;
		v[i] = (v[i])/sig ;
	}
}

void standart_Z(DT ** v,int n,int p,DT * isna,DT * mean,DT * sig) {

	for (int j=0; j<p; j++) {
		mean[j]=0;
		sig[j]=0;
		isna[j] = 1.0;
	}

	for (int i=0; i<n; i++) {
		for (int j=0; j<p; j++) {
			mean[j] += v[i][j] ;
		}
	}

	DT invn=1.0/n;
	DT * sv = new DT[p];

	for (int j=0; j<p; j++) {
		mean[j] *= invn;
		sv[j] = 0;
	}

	for (int i=0; i<n; i++){
		for (int j=0; j<p; j++) {
			v[i][j] = (v[i][j] - mean[j]);
			sv[j] += v[i][j]*v[i][j];
		}
	}

	invn=1.0/(n-1);
	for (int j=0; j<p; j++) {
		sig[j] = sqrt((sv[j]*invn)) ;
		if (sig[j] == 0) {
			isna[j] = 0.0;
			sig[j] = 1.0;
		}
	}

	delete[] sv ;

	for (int i=0; i<n; i++){
		for (int j=0; j<p; j++) {
			if ( isna[j] != 0.0 ) {
				v[i][j] = v[i][j]/sig[j] ;
			}
		}
	}
}

void standart2(DT ** v,int n,int ip,DT &isna,DT &mean,DT &sig)
{

	isna=1.0;
	//mean
	double sv=0;
	mean=0,sig=0;
	for (int i=0; i<n; i++)
	{
		mean = mean + v[i][ip];
		//sv = sv + v[i]*v[i];
	}

	mean = mean/(n);
	// cout << "mean:"<<mean<< endl;
	for (int i=0; i<n; i++)
	{
		v[i][ip] = (v[i][ip] - mean);
		sv = sv + v[i][ip]*v[i][ip];
	}

	//sig= sqrt((sv/(n)) - mean*mean) ;
	sig = sqrt((sv/(n-1))) ;
	//sig=1;
	//manage NA value
	if ( sig == 0 )
	{
		isna=0.0;
		sig=1;
		return;
	}
	//standart core
	for (int i=0; i<n; i++)
	{
		// v[i] = (v[i] - mean)/sig ;
		v[i][ip] = (v[i][ip])/sig ;
	}
}



/*
     TODO , ofilangi : debug ....importer du fortran mais jamais utiliser
 */
/*
subroutine ginv1(a,n,m,tol,irank)
returns generalized inverse of matrix x of size n x n declared
as m x m. tol is working zero and irank returns the rank of
the matrix. w is a work vector of size m,
by rohan fernando, slightly structured by i. misztal 05/05/87

ofilangi : using c index: column + row*NUMCOL
 */

void ginv(DT *a,int n,int m,DT tol,int &irank)
{
	cout << "ginv n="<< n << endl ;
	double w[n],re,sum;
	int iim1;

	irank=n;
	for (int i=0; i<n; i++)
	{
		cout << "i:"<< i << ",i-1="<< i-1<< endl ;
		for (int j=0; j<i-1; j++)
		{
			re = a[j+i*m];
			cout << "re:" << re << endl ;
			for (int ii=i-1; ii<n; ii++)
			{
				a[i+ii*m]=a[i+ii*m]-re*a[i+ii*m];
			}
		}
		cout << "1:"<< a[i+i*m] << " "<< tol<< endl ;
		if ( a[i+i*m]<tol)
		{
			cout << "inf.."<< endl;
			a[i+i*m]=0;
			for (int ii=i+1; ii<n; ii++)
			{
				a[i+ii*m]=0;
			}
			irank = irank-1;
		}
		else
		{
			a[i+i*m]=sqrt(a[i+i*m]);
			cout << "else:"<< a[i+i*m] << endl;
			for (int ii=i+1; ii<n; ii++)
			{
				a[i+ii*m]=a[i+ii*m]/a[i+i*m];
				cout << "2:a["<<ii<<"]["<<i<<"]="<< a[i+ii*m] << endl ;
			}
		}
	}

	for (int i=0; i<n; i++)
	{
		if (a[i+i*m] == 0 )
		{
			for (int ii=i+1; ii<n; ii++)
			{
				a[i+ii*m]=0;
			}
		}
		else
		{
			a[i+i*m]=1/a[i+i*m];
			for (int ii=i+1; ii<n; ii++)
			{
				w[ii]=0;
			}
			for (int ii=i+1; ii<n; ii++)
			{
				iim1=ii-1;
				re=a[i+iim1*m];
				for (int iii=ii; iii<n; iii++)
				{
					w[iii]-=a[iim1+iii*m]*re;
				}
				if ( a[ii+ii*m] == 0 )
				{
					a[i+ii*m]=0;
				}
				else
				{
					a[i+ii*m] = w[ii]/a[ii+ii*m];
				}
			}
		}
	}
	for (int j=0; j<n; j++)
	{
		for (int i=j; i<n; i++)
		{
			sum=0;
			for (int ii=i; ii<n; ii++)
			{
				sum+=a[j+ii*m]*a[i+ii*m];
			}
			a[j+i*m]=sum;
		}
	}

	for (int i=0; i<n; i++)
	{
		for (int j=i+1; j<n; j++)
		{
			//a[i+j*m] = a[j+i*m];
		}
	}
}

void exitWithError(const std::string &error)
{
	std::cout << error << std::endl ;
	//	std::cin.ignore();
	//	std::cin.get();

	exit(EXIT_FAILURE);
}


