/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
#ifndef COMMON_HPP_INCLUDED
#define COMMON_HPP_INCLUDED

#ifdef HAVE_CONFIG_H
#include "config.h"
#else
#define DT double
#define DT_DOUBLE true
#endif

#include <cstdlib>
#include <string>
#include <algorithm>


#if (DT_DOUBLE)
#define getrf dgetrf_
#define getri dgetri_
#define gesv dgesv_
#define nrm2 dnrm2_
#define dot ddot_
#define gemv dgemv_
#else
#define getrf sgetrf_
#define getri sgetri_
#define gesv sgesv_
#define nrm2 snrm2_
#define dot sdot_
#define gemv sgemv_
#endif

/* LAPACK Dependences */
extern "C" {
/* LU Decomposition */
void dgetrf_(int *m, int *n, double *a, int *lda, int *ipiv, int *info);
/* Inversion matrix from LU decomposition*/
void dgetri_(int *m, double *, int *lda, int *ipiv, double *work, int *lwork, int *info);
/* solve A.X = B */
void dgesv_( int* n, int* nrhs, double* a, int* lda, int* ipiv,double* b, int* ldb, int* info );

/* BLAS norm euclidienne */
double dnrm2_(int *n, double *x, int *incx);
/* BLAS Dot product */
double ddot_(int *n,double *dx, int* incx,int *dy, int *incy);

/* BLAS 2 matrix-vector multiplication y := alpha*A*x + beta*y */
void dgemv_(char* t,int *m, int *n, double *alpha, double *a,int *lda,double *x, int * incx, double *beta, double *y,int *incy);

/* Simple precision */

/* LU Decomposition */
void sgetrf_(int *m, int *n, float *a, int *lda, int *ipiv, int *info);
/* Inversion matrix from LU decomposition*/
void sgetri_(int *m, float *, int *lda, int *ipiv, float *work, int *lwork, int *info);
/* solve A.X = B */
void sgesv_( int* n, int* nrhs, float* a, int* lda, int* ipiv,float* b, int* ldb, int* info );

/* BLAS norm euclidienne */
float snrm2_(int *n, float *x, int *incx);
/* BLAS Dot product */
float sdot_(int *n,float *dx, int* incx,int *dy, int *incy);

/* BLAS 2 matrix-vector multiplication y := alpha*A*x + beta*y */
void sgemv_(char* t,int *m, int *n, float *alpha, float *a,int *lda,float *x, int * incx, float *beta, float *y,int *incy);
}


/* standardise le tableau V dim=1, rend la moyenne et l'ecart type*/
void standart1(DT * v,int n,DT &mean,DT &sig);

/* standardise le tableau V dim=2 sur la colonne ip, rend la moyenne et l'ecart type*/
void standart2(DT ** v,int n,int ip,DT &isna,DT &mean,DT &sig);

/* optimisation de la fonction standart2 */
void standart_Z(DT ** v,int n,int p,DT * isna,DT * mean,DT * sig);

/* calcul la moyenne d un tableau dim=1 */
DT calmean(DT * v,int n);

/* calcul l'ecart type d'un tableau dim=1 */
DT calsig(DT * v,int n,const DT mean);

/* Exit with a error message on the console */
void exitWithError(const std::string &error);

// trim from start
static inline std::string &ltrim(std::string &s) {
        s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, int>(std::isspace))));
        return s;
}

// trim from end
static inline std::string &rtrim(std::string &s) {
        s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
        return s;
}

// trim from both ends
static inline std::string &trim(std::string &s) {
        return ltrim(rtrim(s));
}

/*
#ifdef TYPE_SP
#define DT float
#else
#define DT double
#endif
 */



#endif // COMMON_HPP_INCLUDED
