/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
#include "common.hpp"
#include "dataGen.hpp"

#include <string>
#include <iostream>
using namespace std;

dataGen::dataGen() {
	n=0;
	p=0;
	Z=0;
	mapY.clear();
	mapEDC.clear();
	phenoId.clear();
	genoId.clear();
	ngenoRecord = 0;
	nphenoRecord = 0;
}

dataGen::~dataGen(){
	this->release();
}

void dataGen::release() {

	for (int i=0;i<n;i++) {
		if (  Z && Z[i] ) {
			delete[] Z[i];
		}
	}

	if ( Z ) delete[] Z;

	for (map<string,DT *>::const_iterator it=mapY.begin();it != mapY.end();it++) {
		delete[] (*it).second;
	}

	for (map<string,DT *>::const_iterator it=mapEDC.begin();it != mapEDC.end();it++) {
			delete[] (*it).second;
	}

	Z=NULL;
	mapY.clear();
	mapEDC.clear();
	phenoId.clear();
	genoId.clear();
}


void dataGen::clone(dataGen &copy,bool genotype) const {

	copy.n = this->n;
	copy.p = this->p;

	if (genotype) copy.Z = new DT*[this->n];

	for (map<string,DT *>::const_iterator it=mapY.begin();it != mapY.end();it++) {
		DT * Y = NULL ;
		if ( (*it).second != NULL ) {
			Y = new DT[this->n];
			for (int i=0;i<this->n;i++) {
				Y[i] = (*it).second[i];
			}
		} else {
			Y = NULL;
		}

		copy.mapY[(*it).first] = Y;
	}

	for (map<string,DT *>::const_iterator it=mapEDC.begin();it != mapEDC.end();it++) {
		DT * EDC = NULL ;
			if ( (*it).second != NULL ) {
				EDC = new DT[this->n];
				for (int i=0;i<this->n;i++) {
					EDC[i] = (*it).second[i];
				}
			} else {
				EDC = NULL;
			}

			copy.mapEDC[(*it).first] = EDC;
		}


	copy.phenoId = phenoId ;
	copy.genoId = genoId ;

	for (int i=0;i<this->n;i++) {
		if (genotype) {
			copy.Z[i] = new DT[this->p];
			for (int j=0;j<this->p;j++) {
				copy.Z[i][j] = this->Z[i][j];
			}
		}
	}
}

