/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
#include "factoryData.hpp"
#include "convert.hpp"

#include <stdio.h>
#include <string.h>

#include <fstream>
#include <string>
#include <vector>
#include <iostream>
#include <algorithm>
using namespace std;

factoryData::factoryData() {

}

factoryData::~factoryData() {

}

int factoryData::buildDataGenPhenotype(const string& pheno,dataGen &data,bool edc) {
	return readRecordPheno(pheno,data,edc);
}

int factoryData::buildDataGenGenotype(const string& geno,dataGen &data,int format,bool headerGeno) {

	void (*funct)(const string&,const int,dataGen&) = NULL;
	int (*functNbMark)(ifstream&) = NULL;

	if (format == AIPL_SPACE) {
		funct = factoryData::addLineMarkerAIPLWithBlanck;
		if ( headerGeno )
			functNbMark = getNumberMarkerWithFirstLineRecordHeader;
		else
			functNbMark = getNumberMarkerWithFirstLineRecord;
	} else if (format == PLINK) {
		funct = factoryData::addLineMarkerPLINK;
		if ( headerGeno )
			functNbMark=factoryData::getNumberMarkerPLINKHeader;
		else
			functNbMark=factoryData::getNumberMarkerPLINK;
	} else if (format == MPE) {
		funct = factoryData::addLineMarkerMPE;
		if ( headerGeno )
			functNbMark=factoryData::getNumberMarkerMPEHeader;
		else
			functNbMark=factoryData::getNumberMarkerMPE;
	} else if (format == AIPL) {
		funct = factoryData::addLineMarkerAIPL;
		if ( headerGeno )
			functNbMark=factoryData::getNumberMarkerAIPLHeader;
		else
			functNbMark=factoryData::getNumberMarkerAIPL;
	} else if (format == UGA) {
		funct = factoryData::addLineMarkerUGA;
		if ( headerGeno )
			functNbMark=factoryData::getNumberMarkerUGAHeader;
		else
			functNbMark=factoryData::getNumberMarkerUGA;
	}
	else {
		cerr << "This genotype format is not yet implemented !"<< endl ;
		exit(1);
	}

	return readRecordGeno(geno,data,headerGeno,functNbMark,funct);

	cerr << "Unknown format file !" << endl ;
	exit(-1);

}


/* init phenotype without header and record name */
int factoryData::readRecordPheno(const string& pheno,dataGen &data,bool edc) {

	ifstream phenoFile;

	phenoFile.open(pheno.c_str());

	if ( ! phenoFile.is_open() )
	{
		cerr << "can not find phenotype file!" << endl ;
		exit(-1);
	}

	string line;
	data.nphenoRecord=getNumberRecord(phenoFile,true) ;

	if ( data.nphenoRecord <= 0 ) {
		cerr << "can not find phenotype record in "<< pheno << endl ;
		exit(-1);
	}

	int npheno = getNumberTokenLine(phenoFile,0) ;

	/*
	 * EDC+PHENO
	 */
	if (edc) npheno /= 2 ;

	cout << "[phenotype]number of phenotype :"<< npheno << endl ;

	getline (phenoFile,line);
	istringstream stream1(line);
	for (int ipheno=0;ipheno < npheno ;ipheno++ ) {
		string nameVarY ;
		string nameVarEDC ;
		if (edc) stream1 >> nameVarEDC;
		stream1 >> nameVarY;
		data.mapY[nameVarY] = new DT[data.nphenoRecord];
		if (edc) data.mapEDC[nameVarY] = new DT[data.nphenoRecord];
	}

	int n=0;
	while ( ! phenoFile.eof() )
	{
		getline (phenoFile,line);
		trim(line);
		if ( line.compare("") != 0 )
		{
			string nameId;
			istringstream stream1(line);
			//record name
			stream1 >> nameId;
			data.phenoId[nameId] = n;

			map<string,DT*>::const_iterator itY=data.mapY.begin();
			map<string,DT*>::const_iterator itEDC=data.mapEDC.begin();

			for (int i=0; i < npheno ;i++) {
				if (edc) stream1 >> (*itEDC).second[n];
				stream1 >> (*itY).second[n];
			}
			n++;
		}
	}

	cout << "[phenotype]number of record    :"<< data.nphenoRecord << endl ;
	phenoFile.close();

	return 0;
}

/* init genotype without header and record name */
int factoryData::readRecordGeno(const string& geno,
		dataGen &data,
		bool header,
		int (*functNbMark)(ifstream&),
		void (*functReadLine)(const string&,const int,dataGen&)) {

	ifstream genoFile;
	genoFile.open(geno.c_str());

	if ( ! genoFile.is_open() )
	{
		cerr << "can not find genotype file!" << endl ;
		exit(1);
	}

	data.p = functNbMark(genoFile);
	cout << "[genotype]nb marker:"<< data.p << endl ;

	if ( data.p == 0 ) {
		cerr << "[genotype]No marker information find..."<< endl ;
		return -1;
	}


	data.ngenoRecord = getNumberRecord(genoFile,header);

	cout << "[genotype]record="<< data.ngenoRecord << endl ;
	data.Z = new DT*[data.ngenoRecord];

	for (int i=0; i<data.ngenoRecord; i++) {
		data.Z[i]=new DT[data.p];
	}

	string line;
	int nx=0;
	bool head=header;
	int lx=0;
	while ( ! genoFile.eof() )
	{
		lx++;
		getline (genoFile,line);
		trim(line);
		if ( line.compare("") != 0 )
		{
			string v;
			if ( head ) {
				head = false;
				continue;
			}
			istringstream stream1(line);
			stream1 >> v ;
			data.genoId[v] = nx;
			line.erase(0,v.length()+1);
			if ( line.compare("") == 0 ) {
				cerr << "can not read genotype line:"<< lx << endl ;
				exit(1);
			}
			functReadLine(line,nx,data);
			nx++;
		}
	}
	cout << "[genotype ] read "<< data.ngenoRecord << " records."<< endl;
	return 0;
}

void factoryData::reorganize_Y_Z(dataGen &data) {

	cout << "build Y and Z..." << endl ;

	map<string,DT *> mapY;
	map<string,DT *> mapEDC;
	DT ** Z ;

	map<string,int> newPhenoId;
	map<string,int> newGenoId;

	if ( data.nphenoRecord > data.ngenoRecord)
		data.n = data.nphenoRecord ;
	else
		data.n = data.ngenoRecord  ;

	Z = new DT*[data.n];

	for (map<string,DT *>::const_iterator it = data.mapY.begin(); it != data.mapY.end();it++) {
		mapY[(*it).first] = new DT [data.n];
	}

	for (map<string,DT *>::const_iterator it = data.mapEDC.begin(); it != data.mapEDC.end();it++) {
		mapEDC[(*it).first] = new DT [data.n];
	}

	int idx = 0 ;
	for (map<string,int>::const_iterator it = data.phenoId.begin();it != data.phenoId.end();it++) {
		string v = (*it).first;
		int idY = (*it).second;
		map<string,int>::const_iterator it2 = data.genoId.find(v);
		if ( it2 == data.genoId.end() ) {
			cerr << "[skip record id= "<< v <<"]"<< endl;
			continue;
		}
		int idZ = (*it2).second;

		Z[idx] = new DT[data.p];
		//		for (int i=0;i<data.p;i++)
		//			Z[idx][i] = data.Z[idZ][i];
		memcpy(Z[idx],data.Z[idZ],data.p*sizeof(DT));

		delete[] data.Z[idZ];
		data.Z[idZ] = NULL;

		for (map<string,DT *>::const_iterator it3 = mapY.begin(); it3 != mapY.end();it3++) {
			DT * Y = mapY[(*it3).first];
			Y[idx] = data.mapY[(*it3).first][idY];
		}
		for (map<string,DT *>::const_iterator it3 = mapEDC.begin(); it3 != mapEDC.end();it3++) {
			DT * EDC = mapEDC[(*it3).first];
			EDC[idx] = data.mapEDC[(*it3).first][idY];
		}
		newPhenoId[v] = idx;
		newGenoId[v] = idx;
		idx++;
	}

	/* remove and replace with new Z */
	for (int i=0;i<data.ngenoRecord;i++) {
		if ( data.Z[i] != NULL ) delete [] data.Z[i];
	}
	delete [] data.Z;
	data.Z = Z;

	for (map<string,DT *>::const_iterator it = data.mapY.begin(); it != data.mapY.end();it++) {
		delete [] data.mapY[(*it).first] ;
	}
	data.mapY = mapY ;

	for (map<string,DT *>::const_iterator it = data.mapEDC.begin(); it != data.mapEDC.end();it++) {
		delete [] data.mapEDC[(*it).first] ;
	}
	data.mapEDC = mapEDC ;

	if ( idx != data.n ) {
		cout << "resize array Z :"<< data.n << " to " << idx << " records" << endl ;
		for (int i=idx;i<data.n;i++) {
			data.Z[idx] = NULL;
		}
		data.n = idx;
	}

	if ( idx == 0 ) {
		cerr << "Phenotype and genotype IDs does no match !" << endl ;
		exit(1);
	}

	data.phenoId = newPhenoId;
	data.genoId = newGenoId;

	factoryData::manage_missingdata_snp(data);

	cout << "ok" << endl ;


}

int factoryData::check_minor_allel_frequency(dataGen &data,float mafmin,int *listSnpMafMin) {
	int n=0;

	if ( data.n > 0 ) {
		n = data.n;
	} else if ( data.ngenoRecord > 0 ) {
		n = data.ngenoRecord;
	}

	if ( n<= 0 ) return 0 ;

	DT * freq2 = new DT[data.p];

	for (int j = 0;j<data.p;j++)
		freq2[j] = 0;

	int * countSNP = new int[data.p];
	for (int j = 0;j<data.p;j++)
		countSNP[j] = 0;

	for ( int i=0; i<n;i++ ) {
		for (int j = 0;j<data.p;j++) {
			if ((data.Z[i][j] >= 0) && (data.Z[i][j]<=2) ) {
				freq2[j] += data.Z[i][j] / 2 ;
				countSNP[j]++;
			}
		}
	}

	int ncountthres=0;
	int nmisSNP=0;
	for (int j = 0;j<data.p;j++) {
		if ( countSNP[j] == 0 ) {
			cerr << "All ID are genotyped with missing data for SNP "<< j << endl ;
			continue;
		}

		if ( countSNP[j] != n ) nmisSNP++;

		freq2[j] /= countSNP[j];
		bool test = freq2[j] < mafmin ;
		test = test || (1 - freq2[j]) < mafmin ;
		if ( test ) {
			if (listSnpMafMin != NULL) {
				listSnpMafMin[ncountthres]=j;
			}
			ncountthres++;
			cout << "[warn maf] snp "<< (j+1) << " freq2:" << freq2[j] << " freq1="<< 1 - freq2[j] << endl ;
		}
	}

	cout << "Number snp under MAF threshold ["<< mafmin << "] ["<< ncountthres<< "]" << endl ;
	cout << "Number of SNP with missing data :"<< nmisSNP << endl ;

	delete [] countSNP;
	delete [] freq2;

	return ncountthres;
}

/*
 * Calcul de la moyenne de la colonne du Jieme SNP pour affecter les valeurs manquantes
 */
void factoryData::manage_missingdata_snp(dataGen &data) {

	cout << "manage data" << endl ;
	DT * mean = new DT[data.p];
	int * countSNP = new int[data.p];

	for (int j = 0;j<data.p;j++) {
		mean[j] = 0;
	}

	for ( int i=0; i<data.n;i++ ) {
		for (int j = 0;j<data.p;j++) {
			if ((data.Z[i][j] >= 0) && (data.Z[i][j]<=2) ) {
				mean[j] += data.Z[i][j];
				countSNP[j]++;
			}
		}
	}

	for (int j = 0;j<data.p;j++) {
		mean[j] /= countSNP[j];
	}
	/*
	 * Affect mean of column for missing data
	 */
	int count=0;
	for ( int i=0; i<data.n;i++ ) {
		for (int j = 0;j<data.p;j++) {
			if ((data.Z[i][j] < 0) | (data.Z[i][j]>2) ) {
				//cout << data.Z[i][j] << " "<< mean[j] << endl ;
				data.Z[i][j] = mean[j] ;
				count++;
			}
		}
	}

	delete[] mean;
	delete[] countSNP;

	cout << "missing genotype data: "<< count << " value(s) affected with the mean of the corresponding column." << endl ;
}

void factoryData::remove_minmaf_snp(dataGen &data,int *listSnp,int nremove,int * listOk) {

	cout << "[remove snp with maf < minmaf]"<< endl;
	int n=0;

	if ( data.n > 0 ) {
		n = data.n;
	} else if ( data.ngenoRecord > 0 ) {
		n = data.ngenoRecord;
	}

	DT ** Z ;

	Z = new DT*[n];
	int p = data.p - nremove ;

	if ( p <= 0) {
		cerr << "devel : can not perform remove_snp new p="<< p << endl ;
		return;
	}

	int isnp=0;
	for (int i=0;i<data.p;i++) {
		int j;
		for ( j=0; j<nremove;j++) {
			if ( i == listSnp[j] ) break;
		}
		if ( j >= nremove ) listOk[isnp++]=i;
	}

	for (int i=0;i<n;i++) {
		Z[i] = new DT[p];
		for (int j=0;j<p;j++) {
			Z[i][j] = data.Z[i][listOk[j]];
		}
	}

	for (int i=0;i<n;i++) {
		delete[] data.Z[i];
	}

	delete[] data.Z ;
	data.Z = Z;
	data.p = p;

	cout << "[ok] nsnp="<< p << endl;
}

/*
 * Give the number of token
 */
int factoryData::getNumberTokenLine(ifstream &file,int lineNumber) {

	if ( ! file.is_open() )
		return 0;
	file.clear();
	file.seekg(0, ios::beg) ;

	string line;

	int iline = 0 ;
	while ( (! file.eof()) && ( iline < lineNumber) ) {
		getline (file,line);
		if ( line.compare("") != 0 ) iline++;
	}

	int n=0;

	if ( ! file.eof() )
	{
		getline (file,line);
		istringstream stream1(line);
		string f;
		while ( stream1 )
		{
			stream1 >> f;
			if ( (stream1.rdstate() & istringstream::failbit) == 0 ) n++;
		}
	}
	file.clear();
	file.seekg(0, ios::beg) ;
	return n;

}

int factoryData::getNumberMarkerWithFirstLineRecord(ifstream &file) {
	return (getNumberTokenLine(file,0) - 1);
}

int factoryData::getNumberMarkerWithFirstLineRecordHeader(ifstream &file) {
	return (getNumberTokenLine(file,1) - 1);
}

int factoryData::getNumberMarkerPLINK(ifstream &file) {
	return ((getNumberTokenLine(file,0) - 6) / 2);
}

int factoryData::getNumberMarkerPLINKHeader(ifstream &file) {
	return ((getNumberTokenLine(file,1) - 6) / 2);
}

int factoryData::getNumberMarkerMPE(ifstream &file) {
	return ((getNumberTokenLine(file,0) - 1) / 2);
}

int factoryData::getNumberMarkerMPEHeader(ifstream &file) {
	return ((getNumberTokenLine(file,1) - 1) / 2);
}

int factoryData::getNumberMarkerAIPL(ifstream &file) {
	file.clear();
	file.seekg(0, ios::beg) ;
	string line;
	getline (file,line);

	istringstream stream1(line);
	string v="";
	if ( stream1 )
	{
		stream1 >> v ; /* record name */
		stream1 >> v ; /* string 100212....*/
	}
	file.clear();
	file.seekg(0, ios::beg) ;
	return (int)v.size();
}

int factoryData::getNumberMarkerAIPLHeader(ifstream &file) {
	file.clear();
	file.seekg(0, ios::beg) ;
	string line;
	getline (file,line); /* header */
	getline (file,line);

	istringstream stream1(line);
	string v="";
	if ( stream1 )
	{
		stream1 >> v ; /* record name */
		stream1 >> v ; /* string 100212....*/
	}
	file.clear();
	file.seekg(0, ios::beg) ;
	return (int)v.size();
}

int factoryData::getNumberMarkerUGA(ifstream &file) {
	file.clear();
	file.seekg(0, ios::beg) ;
	string line;
	getline (file,line);

	istringstream stream1(line);
	string v="";
	if ( stream1 )
	{
		stream1 >> v ; /* record name */
		stream1 >> v ; /* nb marker */
	}
	file.clear();
	file.seekg(0, ios::beg) ;

	return Convert::string_to_T<int>(v);
}

int factoryData::getNumberMarkerUGAHeader(ifstream &file) {
	file.clear();
	file.seekg(0, ios::beg) ;
	string line;
	getline (file,line); /* header */
	getline (file,line);

	istringstream stream1(line);
	string v="";
	if ( stream1 )
	{
		string v;
		stream1 >> v ; /* record name */
		stream1 >> v ; /* nb marker */

	}
	file.clear();
	file.seekg(0, ios::beg) ;

	return Convert::string_to_T<int>(v);
}

/*
 * Give the number of records
 */
int factoryData::getNumberRecord(ifstream &file,bool header) {
	if ( ! file.is_open() )
		return 0;

	file.clear();
	file.seekg(0, ios::beg) ;
	int n = 0;
	/* lecture du header */
	if ( header ) {
		bool ko = true ;
		while ( ko ) {
			if ( file.eof() ) {
				cerr << "can not read line." << endl ;
				exit(1);
			}
			string line;
			getline (file,line);
			if ( line.compare("") != 0 ){
				int nend = 25;
				if ( line.size() < 25 ) {
					nend=line.size();
					cout << "header:"<< line.substr(0,nend)<< endl ;
				} else
					cout << "header:"<< line.substr(0,nend)<<"..." << endl ;
				ko = false;
			}
		}
	}
	while ( ! file.eof() )
	{
		string line;
		getline (file,line);
		if ( line.compare("") != 0 ) n++;
	}
	file.clear();
	file.seekg(0, ios::beg) ;
	return n;
}

/*
 * Attention le premier champ (ID) est deja enleve !!!!
 * Add Information marker separate with blank
 */
void factoryData::addLineMarkerAIPLWithBlanck(const string &line,const int idx,dataGen & data) {

	istringstream stream1(line);
	int ny=0;

	while ( stream1 )
	{
		double v;
		stream1 >> v ;

		if ( (stream1.rdstate() & istringstream::failbit) == 0 )
		{
			data.Z[idx][ny++] = v ;
		}
	}
}
/*
 * Attention le premier champ (ID) est deja enleve !!!!
 */
void factoryData::addLineMarkerAIPL(const string &line,const int idx,dataGen & data) {

	istringstream stream1(line);
	int ny=0;

	if ( stream1 )
	{
		string v;
		stream1 >> v ;
		if ( (stream1.rdstate() & istringstream::failbit) == 0 )
		{
			for (unsigned i=0;i<v.size();i++) { /* no deals with missing data */
				data.Z[idx][ny++] = v[i] - '0' ;
			}
		}
	}
}
/*
 * Attention le premier champ (ID) est deja enleve !!!!
 */
void factoryData::addLineMarkerUGA(const string &line,const int idx,dataGen & data) {

	istringstream stream1(line);
	int ny=0;

	if ( stream1 )
	{
		unsigned int size ;
		stream1 >> size ;
		string v;
		stream1 >> v ;
		if ( (stream1.rdstate() & istringstream::failbit) == 0 )
		{
			if ( size != v.size() ) {
				cerr << "[UGA format] number of marker is not conform ! ["<< size<< "][" << v.size() << "]"<< endl ;
				exit(1);
			}

			for (unsigned i=0;i<v.size();i++) { /* no deals with missing data */
				data.Z[idx][ny++] = v[i] - '0' ;
			}
		}
	}
}
/*
 * Attention le premier champ (ID) est deja enleve !!!!
 */
void factoryData::addLineMarkerPLINK(const string &line,const int idx,dataGen & data) {

	//	cout << "addLineMarkerPLINKWithBlanck" << endl ;
	int ny=0;

	istringstream stream1(line);
	//remove the 5th elements
	string v;
	for (int i=0;i<5;i++)
		stream1 >> v ;

	while ( stream1 )
	{
		//read pair of allele
		string a1,a2;
		stream1 >> a1 ;
		stream1 >> a2 ;
		if ( (stream1.rdstate() & istringstream::failbit) == 0 )
		{
			if ( a1 != a2 ) {
				data.Z[idx][ny++] = 1 ;
			} else if ( a1[0] == 'A' || a1[0] == 'G' ) {
				data.Z[idx][ny++] = 0 ;
			} else {
				data.Z[idx][ny++] = 2 ;
			}
			//	cout << "a1="<< a1 << " a2="<< a2 << "=>"<< data.Z[idx][ny-1] << endl ;
		}
	}
	//exit(1);
}

/*
 * Attention le premier champ (ID) est deja enleve !!!!
 */
void factoryData::addLineMarkerMPE(const string &line,const int idx,dataGen & data) {


	//cout << "addLineMarkerMPEWithBlanck" << endl ;
	int ny=0;

	istringstream stream1(line);

	while ( stream1 )
	{
		//read pair of allele 1 1, 1 2, 2 2
		int a1,a2;
		stream1 >> a1 ;
		stream1 >> a2 ;
		//cout << a1 << " "<< a2 << ",";
		if ( (stream1.rdstate() & istringstream::failbit) == 0 )
		{
			int v = a1+a2;
			if (v>=2 && v<=4)
				data.Z[idx][ny++] = abs(2 - v)     ;
			else
				data.Z[idx][ny++] = 5;
			//	cout << " a1="<< a1 << " a2="<< a2 << " " << abs(2 - (a1+a2)) << endl;
		}
	}
	//	cout << endl;
	//	exit(1);
}

