/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */

#ifndef FACTORYDATA_HPP_INCLUDED
#define FACTORYDATA_HPP_INCLUDED
#include <iostream>

#include "common.hpp"
#include "dataGen.hpp"


class factoryData {

public:

    const static int AIPL        = 0 ;
    const static int PLINK       = 1 ;
    const static int UGA         = 2 ;
    const static int AIPL_SPACE  = 3 ;
    const static int MPE         = 4 ;

    factoryData();
    virtual ~factoryData();
    static int buildDataGenPhenotype(const string& pheno,dataGen &data,bool edc) ;
    static int buildDataGenGenotype(const string& geno,dataGen &data,int format=factoryData::AIPL_SPACE,bool headerGeno=false) ;

    static void reorganize_Y_Z(dataGen &data);
    /* check allele minor frequency */
    static int check_minor_allel_frequency(dataGen &data,float mafmin=0.03, int *listSnpMafMin=NULL);

    static void remove_minmaf_snp(dataGen &data,int *listSnp,int nremove,int *listOk);

    static void manage_missingdata_snp(dataGen &data);

private:

    /* init phenotype with record name */
    static int readRecordPheno(const string& pheno,dataGen &data,bool edc) ;
    /* init genotype with record name */
    static int readRecordGeno(const string& geno,
    						dataGen &data,
    						bool header,
    						int (*functNbMark)(ifstream&),
    						void (*funct)(const string&,const int,dataGen&)) ;

    /* utils */
    /* number of token for a given line number */
    static int getNumberTokenLine(std::ifstream & file,int lineNumber);

    static int getNumberMarkerWithFirstLineRecord(ifstream &file);
    static int getNumberMarkerPLINK(ifstream &file);
    static int getNumberMarkerMPE(ifstream &file);
    static int getNumberMarkerAIPL(ifstream &file);
    static int getNumberMarkerUGA(ifstream &file);

    static int getNumberMarkerWithFirstLineRecordHeader(ifstream &file);
    static int getNumberMarkerPLINKHeader(ifstream &file);
    static int getNumberMarkerMPEHeader(ifstream &file);
    static int getNumberMarkerAIPLHeader(ifstream &file);
    static int getNumberMarkerUGAHeader(ifstream &file);



    /* number of records (line no empty) in a file */
    static int getNumberRecord(std::ifstream &file,bool header) ;
    /* Add Information marker separate with blank */
    static void addLineMarkerAIPLWithBlanck(const string &line,const int idx,dataGen & data);
    static void addLineMarkerPLINK(const string &line,const int idx,dataGen & data);
    static void addLineMarkerMPE(const string &line,const int idx,dataGen & data);
    static void addLineMarkerAIPL(const string &line,const int idx,dataGen & data);
    static void addLineMarkerUGA(const string &line,const int idx,dataGen & data);



};

#endif // FACTORYDATA_HPP_INCLUDED
