/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
#include <iostream>
#include <fstream>
#include <ctime>
#include <string>
using namespace std;

#include "common.hpp"
#include "convert.hpp"
#include "factoryData.hpp"

#include "command.hpp"

#include "cli.hpp"

int main(int argc,char* argv[])
{
	cli cliap(argc,argv);

	if ( argc <= 1 ) {
		cli::print_help(argv[0]);
		return 0;
	}

	if ( argc >= 2 ) {
		if ( cliap.keyExist("--help") ) {
			cli::print_help(argv[0]);
			return 0;
		}
	}

	string formatGeno;

	int maxiter = 100 ;
	double tol = 0.00001;
	string binprefix="";
	bool edc_present=false;
	bool edc_calcul=false;
	bool headerGeno=false;

	if (cliap.keyExist("--maxiter")) {
		maxiter=cliap.getValue<int>("--maxter");
	}
	if (cliap.keyExist("--tol")) {
		tol=cliap.getValue<double>("--tol");
	}
	if (cliap.keyExist("--bin-prefix")) {
		binprefix=cliap.getValue<string>("--bin-prefix");
	}
	if (cliap.keyExist("--format-geno")) {
		formatGeno=cliap.getValue<string>("--format-geno");
	}

	if (cliap.keyExist("--header-geno")) {
		headerGeno=true;
	}

	if (cliap.keyExist("--edc")) {
		edc_present=true;
		if (! cliap.keyExist("--not-use-edc")) {
			edc_calcul=true;
		}
	}

	int codeFormatGeno = factoryData::AIPL_SPACE;
	if ( formatGeno != "" ) {
		if ( formatGeno == "aipl" ) {
			codeFormatGeno = factoryData::AIPL;
		} else if (formatGeno == "uga" ) {
			codeFormatGeno = factoryData::UGA;
		} else if (formatGeno == "mpe" ) {
			codeFormatGeno = factoryData::MPE;
		} else if (formatGeno == "plink" ) {
			codeFormatGeno = factoryData::PLINK;
		} else if (formatGeno == "aipl_space" ) {
			codeFormatGeno = factoryData::AIPL_SPACE;
		}
	}

	clock_t t = clock();

	for (int iarg=1;iarg<argc;iarg++) {
		string opt = argv[iarg];
		/*
		 * Appel option Sparse PLS
		 * Generation du binaire pour lancer le calcul des EBV/VIP/Predict
		 */
		if ( opt == "--spls" ) {
			cliap.check_with_error("--pheno", "give a validation phenotype file !");
			cliap.check_with_error("--geno", "give a validation genotype file !");
			cliap.check_with_error("--format-geno", "give the format of the genotype file [aipl|aipl_space|mpe|uga|plink]!");
			cliap.check_with_error("--ncomp", "--ncomp option missing.");

			DT mafmin=0.01;
			bool removeMinMaf = cliap.keyExist("--remove-minmaf");
			if ( removeMinMaf ) {
				mafmin = cliap.getValue<DT>("--remove-minmaf");
				cout << "Remove marker with MAF<"<<  mafmin << endl ;
			}

			int ncomp = cliap.getValue<int>("--ncomp");

			int nkeep = cliap.getValue<int>("--spls");

			string phenotypeFile_ap=cliap.getValue<string>("--pheno");
			string genotypeFile_ap=cliap.getValue<string>("--geno");


			Command::command_spls(nkeep,ncomp,maxiter,tol,phenotypeFile_ap,genotypeFile_ap,binprefix,
					edc_present,edc_calcul,codeFormatGeno,headerGeno,mafmin,removeMinMaf);

		} else if ( opt == "--pls" ) {
			cliap.check_with_error("--pheno", "give a validation phenotype file !");
			cliap.check_with_error("--geno", "give a validation genotype file !");
			cliap.check_with_error("--format-geno", "give the format of the genotype file [aipl|aipl_space|mpe|uga|plink]!");
			cliap.check_with_error("--ncomp", "--ncomp option missing.");

			DT mafmin=0.01;
			bool removeMinMaf = cliap.keyExist("--remove-minmaf");
			if ( removeMinMaf ) {
				mafmin = cliap.getValue<DT>("--remove-minmaf");
				cout << "Remove marker with MAF<"<<  mafmin << endl ;
			}

			int ncomp = cliap.getValue<int>("--ncomp");

			string phenotypeFile_ap=cliap.getValue<string>("--pheno");
			string genotypeFile_ap=cliap.getValue<string>("--geno");

			Command::command_pls(ncomp,maxiter,tol,phenotypeFile_ap,genotypeFile_ap,binprefix,
					edc_present,edc_calcul,codeFormatGeno,headerGeno,mafmin,removeMinMaf);
		} else if ( opt == "--vip") {

			cliap.check_with_error("--binary", "give a binary file provided by --spls or --pls command!");
			cliap.check_with_error("--out", "give a result file name !");
			cliap.check_with_error("--H", "--H option missing.");

			string binary=cliap.getValue<string>("--binary");
			int H = cliap.getValue<int>("--H");
			string nameOutFile = cliap.getValue<string>("--out");

			Command::command_vip(binary,H,nameOutFile);

			cout << "== Plot des VIP ==" << endl ;
			cout << "R -e \"png(file='"<< nameOutFile+".png" << "',width=800,res=120);VIP<-read.table('"<< nameOutFile << "',header=F,na.strings=\"-9999\");plot(abs(VIP[,"<< H << "]))\"" << endl;

		} else if ( opt == "--valid") {
			/* Check options */
			cliap.check_with_error("--pheno", "give a validation phenotype file !");
			cliap.check_with_error("--geno", "give a validation genotype file !");
			cliap.check_with_error("--format-geno", "give the format of the genotype file [aipl|aipl_space|mpe|uga|plink]!");
			cliap.check_with_error("--binary", "give a binary file provided by --spls or --pls command!");

			string phenotypeFile_val=cliap.getValue<string>("--pheno");
			string genotypeFile_val=cliap.getValue<string>("--geno");
			string binary=cliap.getValue<string>("--binary");

			Command::command_valid(phenotypeFile_val,genotypeFile_val,binary,edc_present,edc_calcul,codeFormatGeno,headerGeno);

		} else if (  opt == "--predict"  ) {
			/* Check options */
			cliap.check_with_error("--geno", "give a genotype file !");
			cliap.check_with_error("--format-geno", "give the format of the genotype file [aipl|aipl_space|mpe|uga|plink]!");
			cliap.check_with_error("--binary", "give a binary file provided by --spls or --pls command!");
			cliap.check_with_error("--H", "--H option missing.");
			cliap.check_with_error("--out", "give a result file name !");

			string genotypeFile_val=cliap.getValue<string>("--geno");
			string binary=cliap.getValue<string>("--binary");
			int H = cliap.getValue<int>("--H");
			string nameOutFile = cliap.getValue<string>("--out");

			Command::command_predict(genotypeFile_val,binary,H,codeFormatGeno,headerGeno,nameOutFile);
		}
	}
	t = clock() - t;
	cout << ">>>>>time="<< ((float)t)/CLOCKS_PER_SEC << endl ;

	return 0;
}

