/*
 *   This file is part of pls4snp.
 *
 *   pls4snp is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   pls4snp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with pls4snp; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Copyright 2013 Olivier Filangi
 *
 */
/*
 * pls4snp.h
 *
 *  Created on: 20 mars 2013
 *      Author: ofilangi
 */

#ifndef PLS4SNP_H_
#define PLS4SNP_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#else
#define DT double
#define DT_DOUBLE true
#endif

/*
 * Interface to call pls/spls from a programme C/C++/fortran
 *
 * Purpose:
 *
 * Parameters:
 *   [in]  n          : specifies number of row of Z and Y
 *   [in]  p          : specifies number of SNP (column) of Z
 *   [in]  Z[n,p]     : incidence matrix
 *   [in]  Y[n]       : performance vector
 *   [in]  edc_pres   : 0 if EDC values are available otherwise >0.
 *   [in]  EDC[n]     : EDC vector
 *   [in]  ncomp      : specifies number of latent variable to build
 *   [in]  maxiter    : maximum number of iteration for convergence
 *   [in]  tol        : criteria for convergence
 *   [in]  keepX      : number of SNP to keep (Sparse PLS option), -1 to perform a PLS.
 *   [out] meanY      : mean of Y
 *   [out] sigmY      : standard deviation of Y
 *   [out] meanZ[p]   : vector of mean of each SNP column
 *   [out] sigmZ[p]   : vector of standard deviation of each SNP column
 *   [out] t[n,ncomp] : latent variable associated to X
 *   [out] u[n,ncomp] : latent variable associated to Y
 *   [out] a[p,ncomp] : loading vectors associated to X
 *   [out] b[ncomp]   : loading vectors associated to Y
 *   [out] c[p,ncomp] : buffer useful for the pedict subroutine
 */

extern "C" void spls_(int * n, int * p, DT * Z, DT * Y, int * edc_pres, DT * EDC, int * ncomp, int * maxiter, DT * tol,int * keepX,
		DT * meanY, DT * sigmY, DT * meanZ, DT * sigmZ, DT * t, DT * u, DT * a, DT * b, DT * c );

/*
 * Interface to call calcul_vip from a programme C/C++/fortran
 *
 * Purpose:
 *
 * Parameters:
 *   [in]  ncomp         : specifies number of latent variable associated with t and a
 *   [in]  hcomp         : specifies number of latent variable to include
 *   [in]  n             : specifies number of row of Y
 *   [in]  p             : specifies number of SNP
 *   [in]  Y[n]          : performance vector
 *   [in]  t[n,ncomp]    : latent variable associated to X
 *   [in]  a[p,ncomp   ] : loading vectors associated to X
 *   [out] VIP[p,ncomp]  : VIP results
 */

extern "C" void  calcul_vip_(int *ncomp,int * hcomp,int *n,int *p, DT* Y, DT * t,DT *a,DT *VIP);

/*
 * Interface to call predict from a programme C/C++/fortran
 *
 * Purpose:
 *
 * Parameters:
 *   [in]  ncomp              : specifies number of latent variable associated with t and a
 *   [in]  n                  : specifies number of row of Y
 *   [in]  p                  : specifies number of SNP
 *   [in]  a[p,ncomp]         : loading vectors associated to X
 *   [in]  c[p,ncomp]         : buffer result from spls subroutine
 *   [in]  b[ncomp]           : loading vectors associated to Y
 *   [in]  meanY              : mean of Y
 *   [in]  sigmY              : standard deviation of Y
 *   [in]  meanZ[p]           : vector of mean of each SNP column
 *   [in]  sigmZ[p]           : vector of standard deviation of each SNP column
 *   [in]  n_pred             : specifies number of row of Z_pred
 *   [in]  Z_pred[n_pred,p]   : incidence matrix to predict
 *   [in]  hcomp              : specifies the number of latente variate to compute yhat,tpred,bhat
 *   [out] yhat[n_pred,hcomp] : Y prediction
 *   [out] tpred[n_pred,hcomp]:
 *   [out] bhat[p,hcomp]      :
 */

extern "C" void predict_(int *n, int *p, int *ncomp, DT * a, DT* c, DT * b,DT * meanY, DT * sigmaY, DT * meanZ, DT* sigmaZ,int * n_pred
		,DT * Z_pred, int * hcomp,DT * yhat,DT * tpred,DT * bhat);

#endif /* PLS4SNP_H_ */
